/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"


// A simple program that opens a read-only connection to a "snapshot" of the
// database at the time the connection is opened.  For more information, see
// the "Version Access" section of Getting Started with Matisse.
int main(int argc, char **argv)
{
  MtDatabase* db;
  NSString *hostname, *dbname;
  
  if (argc < 3) {
    printf("Usage: %s <host> <database>\n", argv[0]);
    return -1;
  }
#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];

  @try {
    db = [(MtDatabase *)[MtDatabase alloc] init : hostname : dbname];

    [db open];

    [db startVersionAccess];

    printf("Successful connection and version access to %s\n",
	   [[db description] UTF8String]);

    [db endVersionAccess];

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }
#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  } // @autoreleasepool
#endif

  printf("Done\n");

  return 0;
}
