/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"

#import "Person.h"
#import "Employee.h"

// A simple example which shows how to use generated code for user classes.
int main(int argc, char **argv)
{
  MtDatabase* db;
  NSString *hostname, *dbname, *searchString;
  
  if (argc < 4) {
    printf("Usage: %s <host> <database> <searchString>\n", argv[0]);
    return -1;
  }
#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];
  searchString = [NSString stringWithCString:argv[3] encoding:NSUTF8StringEncoding];

  @try {

    // The third argument to MtDatabase init is given so that the connection 
    // object can find the mapping between the Objective-C class Person and 
    // the schema class Person defined in the "examples.objc_examples.chaps_6_7_8"
    // namespace.
    //
    MtDynamicObjectFactory *factory = [[MtDynamicObjectFactory alloc] init:@"" :@"examples.objc_examples.chaps_6_7_8"];
    db = [(MtDatabase *)[MtDatabase alloc] init:hostname :dbname andFactory:factory];

    [db open];

    [db startTransaction];
    printf("\nCreating objects...\n");
    // create a new Person
    Person* p1 = [[Person alloc] init:db];
    // modify attributes
    [p1 setFirstName:@"John"];
    [p1 setLastName:@"Jones"];
    [p1 setComment:@"weak needs have always hindered reality"];

    Person* p2 = [[Person alloc] init:db];
    // modify attributes
    [p2 setFirstName:@"Fred"];
    [p2 setLastName:@"Jones"];
    [p2 setComment:@"in reality weak knees hindered many"];

    printf("2 objects with comments:\n");
    printf("%s %s says '%s'\n", [[p1 firstName] UTF8String], 
	   [[p1 lastName] UTF8String], [[p1 comment] UTF8String]);
    printf("%s %s says '%s'\n", [[p2 firstName] UTF8String], 
	   [[p2 lastName] UTF8String], [[p2 comment] UTF8String]);

    [db commit];

    [db startVersionAccess];
    printf("\nLooking for : %s\n",  [searchString UTF8String]);

    Person* x;
    int hits = 0;
    MtObjectEnumerator* piter = [Person commentDictEnumerator:db :searchString :nil :MT_MAX_PREFETCHING];
    while ((x = (Person*)[piter nextObject]) != nil) {
      // show attributes and name of class
      printf("%s %s says '%s'\n", 
	     [[x firstName] UTF8String],
	     [[x lastName] UTF8String],
	     [[x comment] UTF8String]);
      hits++;
    }
    [piter close];
    printf("\n%d comment fields contained '%s' \n", hits, [searchString UTF8String]);

    MtSize num = [[Person getCommentDictDictionary:db] getObjectNumber:searchString :nil];
    printf("\n%d matching object(s)\n", num);

    [db endVersionAccess];


    [db startTransaction:MT_MAX_TRAN_PRIORITY];
    printf("\nRemoving created objects...\n");
    [p1 remove];
    [p2 remove];

    [db commit];

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }
#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  } // @autoreleasepool
#endif

  printf("Done\n");

  return 0;
}
